@@ -10,15 +10,16 @@ from itertools import count
 from operator import itemgetter
 import re
 
-from cssutils.css import CSSStyleSheet, CSSRule, Property
+from css_parser.css import CSSStyleSheet, CSSRule, Property
 
 from css_selectors import Select, INAPPROPRIATE_PSEUDO_CLASSES, SelectorError
 from calibre import as_unicode
 from calibre.ebooks.css_transform_rules import all_properties
-from calibre.ebooks.oeb.base import OEB_STYLES, XHTML
+from calibre.ebooks.oeb.base import OEB_STYLES, XHTML, css_text
 from calibre.ebooks.oeb.normalize_css import normalizers, DEFAULTS
 from calibre.ebooks.oeb.stylizer import media_ok, INHERITED
 from tinycss.fonts3 import serialize_font_family, parse_font_family
+from polyglot.builtins import iteritems, itervalues
 
 _html_css_stylesheet = None
 
@@ -45,7 +46,8 @@ def iterrules(container, sheet_name, rules=None, media_rule_ok=media_allowed, ru
     :param sheet_name: The name of the sheet in the container (in case of inline style sheets, the name of the html file)
     :param media_rule_ok: A function to test if a @media rule is allowed
     :param rule_index_counter: A counter object, rule numbers will be calculated by incrementing the counter.
-    :param rule_type: Only yield rules of this type, where type is a string type name, see cssutils.css.CSSRule for the names (by default all rules are yielded)
+    :param rule_type: Only yield rules of this type, where type is a string type name, see css_parser.css.CSSRule for the names (
+                    by default all rules are yielded)
     :return: (CSSRule object, the name of the sheet from which it comes, rule index - a monotonically increasing number)
     '''
 
@@ -82,6 +84,7 @@ def iterrules(container, sheet_name, rules=None, media_rule_ok=media_allowed, ru
 
     importing.discard(sheet_name)
 
+
 StyleDeclaration = namedtuple('StyleDeclaration', 'index declaration pseudo_element')
 Specificity = namedtuple('Specificity', 'is_style num_id num_class num_elem rule_index')
 
@@ -97,13 +100,13 @@ def iterdeclaration(decl):
         if n is None:
             yield p
         else:
-            for k, v in n(p.name, p.propertyValue).iteritems():
+            for k, v in iteritems(n(p.name, p.propertyValue)):
                 yield Property(k, v, p.literalpriority)
 
 
 class Values(tuple):
 
-    ''' A tuple of `cssutils.css.Value ` (and its subclasses) objects. Also has a
+    ''' A tuple of `css_parser.css.Value ` (and its subclasses) objects. Also has a
     `sheet_name` attribute that is the canonical name relative to which URLs
     for this property should be resolved. '''
 
@@ -117,8 +120,8 @@ class Values(tuple):
     def cssText(self):
         ' This will return either a string or a tuple of strings '
         if len(self) == 1:
-            return self[0].cssText
-        return tuple(x.cssText for x in self)
+            return css_text(self[0])
+        return tuple(css_text(x) for x in self)
 
 
 def normalize_style_declaration(decl, sheet_name):
@@ -126,7 +129,7 @@ def normalize_style_declaration(decl, sheet_name):
     for prop in iterdeclaration(decl):
         if prop.name == 'font-family':
             # Needed because of https://bitbucket.org/cthedot/cssutils/issues/66/incorrect-handling-of-spaces-in-font
-            prop.propertyValue.cssText = serialize_font_family(parse_font_family(prop.propertyValue.cssText))
+            prop.propertyValue.cssText = serialize_font_family(parse_font_family(css_text(prop.propertyValue)))
         ans[prop.name] = Values(prop.propertyValue, sheet_name, prop.priority)
     return ans
 
@@ -154,7 +157,7 @@ def resolve_pseudo_declarations(decls):
     groups = defaultdict(list)
     for d in decls:
         groups[d.pseudo_element].append(d)
-    return {k:resolve_declarations(v) for k, v in groups.iteritems()}
+    return {k:resolve_declarations(v) for k, v in iteritems(groups)}
 
 
 def resolve_styles(container, name, select=None, sheet_callback=None):
@@ -163,7 +166,7 @@ def resolve_styles(container, name, select=None, sheet_callback=None):
     style_map = defaultdict(list)
     pseudo_style_map = defaultdict(list)
     rule_index_counter = count()
-    pseudo_pat = re.compile(ur':{1,2}(%s)' % ('|'.join(INAPPROPRIATE_PSEUDO_CLASSES)), re.I)
+    pseudo_pat = re.compile(u':{1,2}(%s)' % ('|'.join(INAPPROPRIATE_PSEUDO_CLASSES)), re.I)
 
     def process_sheet(sheet, sheet_name):
         if sheet_callback is not None:
@@ -216,14 +219,15 @@ def resolve_styles(container, name, select=None, sheet_callback=None):
             style_map[elem].append(StyleDeclaration(Specificity(1, 0, 0, 0, 0), normalize_style_declaration(style, name), None))
 
     for l in (style_map, pseudo_style_map):
-        for x in l.itervalues():
+        for x in itervalues(l):
             x.sort(key=itemgetter(0), reverse=True)
 
-    style_map = {elem:resolve_declarations(x) for elem, x in style_map.iteritems()}
-    pseudo_style_map = {elem:resolve_pseudo_declarations(x) for elem, x in pseudo_style_map.iteritems()}
+    style_map = {elem:resolve_declarations(x) for elem, x in iteritems(style_map)}
+    pseudo_style_map = {elem:resolve_pseudo_declarations(x) for elem, x in iteritems(pseudo_style_map)}
 
     return partial(resolve_property, style_map), partial(resolve_pseudo_property, style_map, pseudo_style_map), select
 
+
 _defvals = None
 
 
@@ -231,7 +235,7 @@ def defvals():
     global _defvals
     if _defvals is None:
         u = type('')
-        _defvals = {k:Values(Property(k, u(val)).propertyValue) for k, val in DEFAULTS.iteritems()}
+        _defvals = {k:Values(Property(k, u(val)).propertyValue) for k, val in iteritems(DEFAULTS)}
     return _defvals
 
 
