@@ -16,6 +16,7 @@ from calibre.ebooks.oeb.base import XHTML, XHTML_NS, barename, namespace, rewrit
 from calibre.ebooks.oeb.stylizer import Stylizer
 from calibre.ebooks import unit_convert
 from calibre.ebooks.textile.unsmarten import unsmarten
+from polyglot.builtins import string_or_bytes
 
 
 class TextileMLizer(OEB2HTML):
@@ -72,9 +73,9 @@ class TextileMLizer(OEB2HTML):
                 # I'm not checking for duplicated spans '%' as any that follow each other were being incorrectly merged
                 txt = '%s' % t
                 if txt != '%':
-                    text = re.sub(r'([^'+t+'|^\n])'+t+'\]\['+t+'([^'+t+'])', r'\1\2', text)
+                    text = re.sub(r'([^'+t+'|^\n])'+t+r'\]\['+t+'([^'+t+'])', r'\1\2', text)
                     text = re.sub(r'([^'+t+'|^\n])'+t+t+'([^'+t+'])', r'\1\2', text)
-                text = re.sub(r'(\s|[*_\'"])\[('+t+'[a-zA-Z0-9 \'",.*_]+'+t+')\](\s|[*_\'"?!,.])', r'\1\2\3', text)
+                text = re.sub(r'(\s|[*_\'"])\[('+t+'[a-zA-Z0-9 \'",.*_]+'+t+r')\](\s|[*_\'"?!,.])', r'\1\2\3', text)
             return text
 
         # Now tidyup links and ids - remove ones that don't have a correponding opposite
@@ -82,13 +83,13 @@ class TextileMLizer(OEB2HTML):
             for i in self.our_links:
                 if i[0] == '#':
                     if i not in self.our_ids:
-                        text = re.sub(r'"(.+)":'+i+'(\s)', r'\1\2', text)
+                        text = re.sub(r'"(.+)":'+i+r'(\s)', r'\1\2', text)
             for i in self.our_ids:
                 if i not in self.our_links:
-                    text = re.sub(r'%?\('+i+'\)\xa0?%?', r'', text)
+                    text = re.sub(r'%?\('+i+'\\)\xa0?%?', r'', text)
 
         # Remove obvious non-needed escaping, add sub/sup-script ones
-        text = check_escaping(text, ['\*', '_', '\*'])
+        text = check_escaping(text, [r'\*', '_', r'\*'])
         # escape the super/sub-scripts if needed
         text = re.sub(r'(\w)([~^]\w+[~^])', r'\1[\2]', text)
         # escape the super/sub-scripts if needed
@@ -111,16 +112,16 @@ class TextileMLizer(OEB2HTML):
 
         # reduce blank lines
         text = re.sub(r'\n{3}', r'\n\np. \n\n', text)
-        text = re.sub(u'%\n(p[<>=]{1,2}\.|p\.)', r'%\n\n\1', text)
+        text = re.sub(u'%\n(p[<>=]{1,2}\\.|p\\.)', r'%\n\n\1', text)
         # Check span following blank para
         text = re.sub(r'\n+ +%', r' %', text)
-        text = re.sub(u'p[<>=]{1,2}\.\n\n?', r'', text)
+        text = re.sub(u'p[<>=]{1,2}\\.\n\n?', r'', text)
         # blank paragraph
         text = re.sub(r'\n(p.*\.)\n', r'\n\1 \n\n', text)
         # blank paragraph
         text = re.sub(u'\n\xa0', r'\np. ', text)
         # blank paragraph
-        text = re.sub(u'\np[<>=]{1,2}?\. \xa0', r'\np. ', text)
+        text = re.sub(u'\np[<>=]{1,2}?\\. \xa0', r'\np. ', text)
         text = re.sub(r'(^|\n)(p.*\. ?\n)(p.*\.)', r'\1\3', text)
         text = re.sub(r'\n(p\. \n)(p.*\.|h.*\.)', r'\n\2', text)
         # sort out spaces in tables
@@ -225,10 +226,10 @@ class TextileMLizer(OEB2HTML):
         '''
 
         # We can only processes tags. If there isn't a tag return any text.
-        if not isinstance(elem.tag, basestring) \
+        if not isinstance(elem.tag, string_or_bytes) \
            or namespace(elem.tag) != XHTML_NS:
             p = elem.getparent()
-            if p is not None and isinstance(p.tag, basestring) and namespace(p.tag) == XHTML_NS \
+            if p is not None and isinstance(p.tag, string_or_bytes) and namespace(p.tag) == XHTML_NS \
                     and elem.tail:
                 return [elem.tail]
             return ['']
