@@ -17,6 +17,8 @@ from css_selectors.errors import ExpressionError
 from css_selectors.parser import parse, ascii_lower, Element
 from css_selectors.ordered_set import OrderedSet
 
+from polyglot.builtins import iteritems, itervalues
+
 PARSE_CACHE_SIZE = 200
 parse_cache = OrderedDict()
 XPATH_CACHE_SIZE = 30
@@ -25,6 +27,7 @@ xpath_cache = OrderedDict()
 # Test that the string is not empty and does not contain whitespace
 is_non_whitespace = re.compile(r'^[^ \t\r\n\f]+$').match
 
+
 def get_parsed_selector(raw):
     try:
         return parse_cache[raw]
@@ -34,6 +37,7 @@ def get_parsed_selector(raw):
             parse_cache.pop(next(iter(parse_cache)))
         return ans
 
+
 def get_compiled_xpath(expr):
     try:
         return xpath_cache[expr]
@@ -43,20 +47,25 @@ def get_compiled_xpath(expr):
             xpath_cache.pop(next(iter(xpath_cache)))
         return ans
 
+
 class AlwaysIn(object):
 
     def __contains__(self, x):
         return True
+
+
 always_in = AlwaysIn()
 
+
 def trace_wrapper(func):
     @wraps(func)
     def trace(*args, **kwargs):
         targs = args[1:] if args and isinstance(args[0], Select) else args
-        print ('Called:', func.__name__, 'with args:', targs, kwargs or '')
+        print('Called:', func.__name__, 'with args:', targs, kwargs or '')
         return func(*args, **kwargs)
     return trace
 
+
 def normalize_language_tag(tag):
     """Return a list of normalized combinations for a `BCP 47` language tag.
 
@@ -78,9 +87,11 @@ def normalize_language_tag(tag):
             taglist.add('-'.join(base_tag + tags))
     return taglist
 
+
 INAPPROPRIATE_PSEUDO_CLASSES = frozenset([
     'active', 'after', 'disabled', 'visited', 'link', 'before', 'focus', 'first-letter', 'enabled', 'first-line', 'hover', 'checked', 'target'])
 
+
 class Select(object):
 
     '''
@@ -142,7 +153,7 @@ class Select(object):
         self.invalidate_caches()
         self.default_lang = default_lang
         if trace:
-            self.dispatch_map = {k:trace_wrapper(v) for k, v in self.dispatch_map.iteritems()}
+            self.dispatch_map = {k:trace_wrapper(v) for k, v in iteritems(self.dispatch_map)}
         if ignore_inappropriate_pseudo_classes:
             self.ignore_inappropriate_pseudo_classes = INAPPROPRIATE_PSEUDO_CLASSES
         else:
@@ -231,7 +242,7 @@ class Select(object):
                 def map_attrib_name(x):
                     return ascii_lower(x.rpartition('}')[2])
             for tag in self.itertag():
-                for attr, val in tag.attrib.iteritems():
+                for attr, val in iteritems(tag.attrib):
                     am[map_attrib_name(attr)][val].add(tag)
         return self._attrib_map
 
@@ -244,7 +255,7 @@ class Select(object):
                 def map_attrib_name(x):
                     return ascii_lower(x.rpartition('}')[2])
             for tag in self.itertag():
-                for attr, val in tag.attrib.iteritems():
+                for attr, val in iteritems(tag.attrib):
                     for v in val.split():
                         am[map_attrib_name(attr)][v].add(tag)
         return self._attrib_space_map
@@ -263,7 +274,7 @@ class Select(object):
                     lang = normalize_language_tag(lang)
                     for dtag in self.itertag(tag):
                         lmap[dtag] = lang
-            for tag, langs in lmap.iteritems():
+            for tag, langs in iteritems(lmap):
                 for lang in langs:
                     lm[lang].add(tag)
         return self._lang_map
@@ -323,6 +334,7 @@ class Select(object):
 
 # Combinators {{{
 
+
 def select_combinedselector(cache, combined):
     """Translate a combined selector."""
     combinator = cache.combinator_mapping[combined.combinator]
@@ -332,6 +344,7 @@ def select_combinedselector(cache, combined):
     for item in cache.dispatch_map[combinator](cache, cache.iterparsedselector(combined.selector), right):
         yield item
 
+
 def select_descendant(cache, left, right):
     """right is a child, grand-child or further descendant of left"""
     right = always_in if right is None else frozenset(right)
@@ -340,6 +353,7 @@ def select_descendant(cache, left, right):
             if descendant in right:
                 yield descendant
 
+
 def select_child(cache, left, right):
     """right is an immediate child of left"""
     right = always_in if right is None else frozenset(right)
@@ -348,6 +362,7 @@ def select_child(cache, left, right):
             if child in right:
                 yield child
 
+
 def select_direct_adjacent(cache, left, right):
     """right is a sibling immediately after left"""
     right = always_in if right is None else frozenset(right)
@@ -357,6 +372,7 @@ def select_direct_adjacent(cache, left, right):
                 yield sibling
             break
 
+
 def select_indirect_adjacent(cache, left, right):
     """right is a sibling after left, immediately or not"""
     right = always_in if right is None else frozenset(right)
@@ -366,6 +382,7 @@ def select_indirect_adjacent(cache, left, right):
                 yield sibling
 # }}}
 
+
 def select_element(cache, selector):
     """A type or universal selector."""
     element = selector.element
@@ -376,6 +393,7 @@ def select_element(cache, selector):
         for elem in cache.element_map[ascii_lower(element)]:
             yield elem
 
+
 def select_hash(cache, selector):
     'An id selector'
     items = cache.id_map[ascii_lower(selector.id)]
@@ -384,6 +402,7 @@ def select_hash(cache, selector):
             if elem in items:
                 yield elem
 
+
 def select_class(cache, selector):
     'A class selector'
     items = cache.class_map[ascii_lower(selector.class_name)]
@@ -392,6 +411,7 @@ def select_class(cache, selector):
             if elem in items:
                 yield elem
 
+
 def select_negation(cache, selector):
     'Implement :not()'
     exclude = frozenset(cache.iterparsedselector(selector.subselector))
@@ -401,6 +421,7 @@ def select_negation(cache, selector):
 
 # Attribute selectors {{{
 
+
 def select_attrib(cache, selector):
     operator = cache.attribute_operator_mapping[selector.operator]
     items = frozenset(cache.dispatch_map[operator](cache, ascii_lower(selector.attrib), selector.value))
@@ -408,44 +429,51 @@ def select_attrib(cache, selector):
         if item in items:
             yield item
 
+
 def select_exists(cache, attrib, value=None):
-    for elem_set in cache.attrib_map[attrib].itervalues():
+    for elem_set in itervalues(cache.attrib_map[attrib]):
         for elem in elem_set:
             yield elem
 
+
 def select_equals(cache, attrib, value):
     for elem in cache.attrib_map[attrib][value]:
         yield elem
 
+
 def select_includes(cache, attrib, value):
     if is_non_whitespace(value):
         for elem in cache.attrib_space_map[attrib][value]:
             yield elem
 
+
 def select_dashmatch(cache, attrib, value):
     if value:
-        for val, elem_set in cache.attrib_map[attrib].iteritems():
+        for val, elem_set in iteritems(cache.attrib_map[attrib]):
             if val == value or val.startswith(value + '-'):
                 for elem in elem_set:
                     yield elem
 
+
 def select_prefixmatch(cache, attrib, value):
     if value:
-        for val, elem_set in cache.attrib_map[attrib].iteritems():
+        for val, elem_set in iteritems(cache.attrib_map[attrib]):
             if val.startswith(value):
                 for elem in elem_set:
                     yield elem
 
+
 def select_suffixmatch(cache, attrib, value):
     if value:
-        for val, elem_set in cache.attrib_map[attrib].iteritems():
+        for val, elem_set in iteritems(cache.attrib_map[attrib]):
             if val.endswith(value):
                 for elem in elem_set:
                     yield elem
 
+
 def select_substringmatch(cache, attrib, value):
     if value:
-        for val, elem_set in cache.attrib_map[attrib].iteritems():
+        for val, elem_set in iteritems(cache.attrib_map[attrib]):
             if value in val:
                 for elem in elem_set:
                     yield elem
@@ -454,6 +482,7 @@ def select_substringmatch(cache, attrib, value):
 
 # Function selectors {{{
 
+
 def select_function(cache, function):
     """Select with a functional pseudo-class."""
     fname = function.name.replace('-', '_')
@@ -472,6 +501,7 @@ def select_function(cache, function):
             if func(cache, function, item):
                 yield item
 
+
 def select_lang(cache, function):
     ' Implement :lang() '
     if function.argument_types() not in (['STRING'], ['IDENT']):
@@ -480,11 +510,12 @@ def select_lang(cache, function):
     if lang:
         lang = ascii_lower(lang)
         lp = lang + '-'
-        for tlang, elem_set in cache.lang_map.iteritems():
+        for tlang, elem_set in iteritems(cache.lang_map):
             if tlang == lang or (tlang is not None and tlang.startswith(lp)):
                 for elem in elem_set:
                     yield elem
 
+
 def select_nth_child(cache, function, elem):
     ' Implement :nth-child() '
     a, b = function.parsed_arguments
@@ -497,6 +528,7 @@ def select_nth_child(cache, function, elem):
     n = (num - b) / a
     return n.is_integer() and n > -1
 
+
 def select_nth_last_child(cache, function, elem):
     ' Implement :nth-last-child() '
     a, b = function.parsed_arguments
@@ -509,6 +541,7 @@ def select_nth_last_child(cache, function, elem):
     n = (num - b) / a
     return n.is_integer() and n > -1
 
+
 def select_nth_of_type(cache, function, elem):
     ' Implement :nth-of-type() '
     a, b = function.parsed_arguments
@@ -521,6 +554,7 @@ def select_nth_of_type(cache, function, elem):
     n = (num - b) / a
     return n.is_integer() and n > -1
 
+
 def select_nth_last_of_type(cache, function, elem):
     ' Implement :nth-last-of-type() '
     a, b = function.parsed_arguments
@@ -537,6 +571,7 @@ def select_nth_last_of_type(cache, function, elem):
 
 # Pseudo elements {{{
 
+
 def select_pseudo(cache, pseudo):
     try:
         func = cache.dispatch_map[pseudo.ident.replace('-', '_')]
@@ -563,50 +598,71 @@ def select_pseudo(cache, pseudo):
         if func(cache, item):
             yield item
 
+
 def select_first_child(cache, elem):
     try:
         return cache.sibling_count(elem) == 0
     except ValueError:
         return False
+
+
 select_first_child.is_pseudo = True
 
+
 def select_last_child(cache, elem):
     try:
         return cache.sibling_count(elem, before=False) == 0
     except ValueError:
         return False
+
+
 select_last_child.is_pseudo = True
 
+
 def select_only_child(cache, elem):
     try:
         return cache.all_sibling_count(elem) == 0
     except ValueError:
         return False
+
+
 select_only_child.is_pseudo = True
 
+
 def select_first_of_type(cache, elem):
     try:
         return cache.sibling_count(elem, same_type=True) == 0
     except ValueError:
         return False
+
+
 select_first_of_type.is_pseudo = True
 
+
 def select_last_of_type(cache, elem):
     try:
         return cache.sibling_count(elem, before=False, same_type=True) == 0
     except ValueError:
         return False
+
+
 select_last_of_type.is_pseudo = True
 
+
 def select_only_of_type(cache, elem):
     try:
         return cache.all_sibling_count(elem, same_type=True) == 0
     except ValueError:
         return False
+
+
 select_only_of_type.is_pseudo = True
 
+
 def select_empty(cache, elem):
     return cache.is_empty(elem)
+
+
 select_empty.is_pseudo = True
 
 # }}}
