@@ -25,9 +25,9 @@ from xml.sax import handler
 from xml.sax.saxutils import escape, quoteattr
 from xml.dom import Node
 
-from opendocument import load
+from .opendocument import load
 
-from namespaces import ANIMNS, CHARTNS, CONFIGNS, DCNS, DR3DNS, DRAWNS, FONS, \
+from .namespaces import ANIMNS, CHARTNS, CONFIGNS, DCNS, DR3DNS, DRAWNS, FONS, \
   FORMNS, MATHNS, METANS, NUMBERNS, OFFICENS, PRESENTATIONNS, SCRIPTNS, \
   SMILNS, STYLENS, SVGNS, TABLENS, TEXTNS, XLINKNS
 
@@ -55,6 +55,7 @@ if False:  # Added by Kovid
 # character etc. styles Since CSS2 has no scope we use a prefix. (Not elegant)
 # In ODF a style can have a parent, these parents can be chained.
 
+
 class StyleToCSS:
 
     """ The purpose of the StyleToCSS class is to contain the rules to convert
@@ -317,6 +318,7 @@ class TagStack:
             if attr in attrs:
                 return attrs[attr]
         return None
+
     def count_tags(self, tag):
         c = 0
         for ttag, tattrs in self.stack:
@@ -324,6 +326,7 @@ class TagStack:
                 c = c + 1
         return c
 
+
 special_styles = {
    'S-Emphasis':'em',
    'S-Citation':'cite',
@@ -352,6 +355,8 @@ special_styles = {
 # ODFCONTENTHANDLER
 #
 # -----------------------------------------------------------------------------
+
+
 class ODF2XHTML(handler.ContentHandler):
 
     """ The ODF2XHTML parses an ODF file and produces XHTML"""
@@ -625,9 +630,6 @@ class ODF2XHTML(handler.ContentHandler):
             self.anchors[name] = "anchor%d" % (len(self.anchors) + 1)
         return self.anchors.get(name)
 
-
-# --------------------------------------------------
-
     def purgedata(self):
         self.data = []
 
@@ -879,7 +881,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
         css_styles = {}
         for name in self.stylestack:
             styles = self.styledict.get(name)
-            css2 = tuple(self.cs.convert_styles(styles).iteritems())
+            css2 = tuple(self.cs.convert_styles(styles).items())
             if css2 in css_styles:
                 css_styles[css2].append(name)
             else:
@@ -900,7 +902,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
                 if k not in ignore:
                     yield k, v
 
-        for css2, names in css_styles.iteritems():
+        for css2, names in css_styles.items():
             self.writeout("%s {\n" % ', '.join(names))
             for style, val in filter_margins(css2):
                 self.writeout("\t%s: %s;\n" % (style, val))
@@ -1192,7 +1194,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
         htmlattrs = {}
         if c:
             htmlattrs['class'] = "TC-%s" % c.replace(".","_")
-        for x in xrange(repeated):
+        for x in range(repeated):
             self.emptytag('col', htmlattrs)
         self.purgedata()
 
@@ -1457,7 +1459,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
 #        self.writeout( escape(mark) )
         # Since HTML only knows about endnotes, there is too much risk that the
         # marker is reused in the source. Therefore we force numeric markers
-        self.writeout(unicode(self.currentnote))
+        self.writeout(type(u'')(self.currentnote))
         self.closetag('a')
         self.closetag('sup')
 
@@ -1566,12 +1568,11 @@ dl.notes dd:last-of-type { page-break-after: avoid }
         self.writedata()
         self.purgedata()
 
-
-# -----------------------------------------------------------------------------
-#
-# Reading the file
-#
-# -----------------------------------------------------------------------------
+    # -----------------------------------------------------------------------------
+    #
+    # Reading the file
+    #
+    # -----------------------------------------------------------------------------
 
     def load(self, odffile):
         """ Loads a document into the parser and parses it.
@@ -1579,8 +1580,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
         """
         self.lines = []
         self._wfunc = self._wlines
-        if isinstance(odffile, basestring) \
-                or hasattr(odffile, 'read'):  # Added by Kovid
+        if isinstance(odffile, (bytes, type(u''))) or hasattr(odffile, 'read'):  # Added by Kovid
             self.document = load(odffile)
         else:
             self.document = odffile
@@ -1593,7 +1593,7 @@ dl.notes dd:last-of-type { page-break-after: avoid }
                 self._walknode(c)
             self.endElementNS(node.qname, node.tagName)
         if node.nodeType == Node.TEXT_NODE or node.nodeType == Node.CDATA_SECTION_NODE:
-            self.characters(unicode(node))
+            self.characters(type(u'')(node))
 
     def odf2xhtml(self, odffile):
         """ Load a file and return the XHTML
